package com.cpp.web.service.station.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cpp.system.service.ISysConfigService;
import com.cpp.web.domain.cloud.ForecastPowerShortTermCloud;
import com.cpp.web.domain.station.*;
import com.cpp.web.mapper.station.PowerStationStatusDataMapper;
import com.cpp.web.service.cloud.ForecastPowerShortTermCloudService;
import com.cpp.web.service.cloud.ForecastPowerUltraShortTermCloudService;
import com.cpp.web.service.cloud.NwpCloudService;
import com.cpp.web.service.station.*;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * cpp_power_station_status_data
 *
 * @author tl
 * @date 2024-09-23 15:28:33
 */
@Service
@RequiredArgsConstructor
public class PowerStationStatusDataServiceImpl extends ServiceImpl<PowerStationStatusDataMapper, PowerStationStatusData> implements PowerStationStatusDataService {
    private final ForecastPowerUltraShortTermStationService forecastPowerUltraShortTermStationService;
    private final ForecastPowerUltraShortTermCloudService forecastPowerUltraShortTermCloudService;
    private final ForecastPowerShortTermCloudService forecastPowerShortTermCloudService;
    private final ElectricFieldService electricFieldService;
//    private final ForecastManufactorService forecastManufactorService;
    private final ForecastPowerShortTermStationService forecastPowerShortTermStationService;
    private final ForecastPowerShortTermRegulationService forecastPowerShortTermRegulationService;
    private final ForecastPowerShortTermSendService forecastPowerShortTermSendService;

    private final WindTowerStatusDataService windTowerStatusDataService;

//    private final SysParameterService sysParameterService;

    private final WeatherStationStatusDataService weatherStationStatusDataService;

    private final NwpCloudService nwpCloudService;

    @Autowired
    private ISysConfigService configService;


    @Override
    public List<PowerStationStatusData> findByStationCodeAndStartTimeAndEndTime(String stationCode, Date startTime, Date endTime) {
        QueryWrapper<PowerStationStatusData> wrapper = new QueryWrapper<>();
        if (stationCode != null && !stationCode.equals("")) {
            wrapper.eq("station_code", stationCode);
        }
        if (startTime != null && !startTime.equals("") && endTime != null && !endTime.equals("")) {
            wrapper.between("time", startTime, endTime);
        }
        return baseMapper.selectList(wrapper);
    }

    public List<ArrayList<String>> timeStamp(Date startTime, Date endTime) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String startDate = sdf.format(startTime);
        String endDate = sdf.format(endTime);
        List<ArrayList<String>> dateList = new ArrayList<>();
        while (startDate.compareTo(endDate) <= 0) {
            List<String> subList = new ArrayList<>();
            subList.add(startDate);
            dateList.add(new ArrayList<>(subList));
            startDate = nextSecond(startDate);
        }
        return dateList;
    }



    @Override
    public Map<String, List> getCenterData(Date startTime, Date endTime) {
        return null;
    }

    public static String nextSecond(String str) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = sdf.parse(str);
        long time = date.getTime() + 1000L * 60 * 15;
        date = new Date(time);
        return sdf.format(date);
    }

    public Map<String, List> getCompositeData(String stationCode, Long startTime, Long endTime) {
        ElectricField electricField = electricFieldService.findByStationCode(stationCode);
        String station = "";

        // 封装列表头信息
        List headList = new ArrayList();
        Map<String,String> czmcHeadMap = new HashMap<>();
        czmcHeadMap.put("field", "czmc");
        czmcHeadMap.put("label", "场站名称");
        headList.add(czmcHeadMap);
        Map<String,String> timeHeadMap = new HashMap<>();
        timeHeadMap.put("field", "time");
        timeHeadMap.put("label", "时间");
        headList.add(timeHeadMap);
        Map<String,String> sjglHeadMap = new HashMap<>();
        sjglHeadMap.put("field", "sjgl");
        sjglHeadMap.put("label", "实际功率");
        headList.add(sjglHeadMap);
        Map<String,String> kyglHeadMap = new HashMap<>();
        kyglHeadMap.put("field", "kygl");
        kyglHeadMap.put("label", "可用功率");
        headList.add(kyglHeadMap);
        Map<String,String> llglHeadMap = new HashMap<>();
        llglHeadMap.put("field", "llgl");
        llglHeadMap.put("label", "理论功率");
        headList.add(llglHeadMap);
        Map<String,String> dqysHeadMap = new HashMap<>();
        dqysHeadMap.put("field", "dqys");
        dqysHeadMap.put("label", "短期原始曲线");
        headList.add(dqysHeadMap);
        Map<String,String> dqzxHeadMap = new HashMap<>();
        dqzxHeadMap.put("field", "dqzx");
        dqzxHeadMap.put("label", "短期中心曲线");
        headList.add(dqzxHeadMap);
        Map<String,String> dqsbHeadMap = new HashMap<>();
        dqsbHeadMap.put("field", "dqsb");
        dqsbHeadMap.put("label", "短期上报曲线");
        headList.add(dqsbHeadMap);

        Map<String,List> tempCdqMap = new HashMap<>();
        Map<String,List> tableCdqMap = new HashMap<>();



        List<String> timeList = new ArrayList<>();
        List<String> sjList = new ArrayList();
        List<String> kyList = new ArrayList();
        List<String> llList = new ArrayList();
        List<String> dqysList = new ArrayList();
        List<String> dqzxList = new ArrayList();
        List<String> dqsbList = new ArrayList();

        // 获取实际功率
        Map<Long, PowerStationStatusData> sjMap = new HashMap<>();
        List<PowerStationStatusData> powerStationStatusDataList = findByStationCodeAndStartTimeAndEndTime(stationCode, new Date(startTime), new Date(endTime));
        for (PowerStationStatusData powerStationStatusData : powerStationStatusDataList) {
            sjMap.put(powerStationStatusData.getTime().getTime(), powerStationStatusData);
        }
        // 获取短期原始曲线
        Map<Long, ForecastPowerShortTermStation> dqysMap = new HashMap<>();
        String dqHowLongAgo = configService.selectConfigByKey("dqHowLongAgo");
        QueryWrapper dqysWrapper = new QueryWrapper<>();
        dqysWrapper.eq("forecast_how_long_ago",Integer.parseInt(dqHowLongAgo));
        dqysWrapper.eq("station_code",stationCode);
        dqysWrapper.between("time", new Date(startTime), new Date(endTime));
        List<ForecastPowerShortTermStation> forecastPowerShortTermStationList =forecastPowerShortTermStationService.list(dqysWrapper);
        for (ForecastPowerShortTermStation forecastPowerShortTermStation : forecastPowerShortTermStationList) {
            dqysMap.put(forecastPowerShortTermStation.getTime().getTime(), forecastPowerShortTermStation);
        }
        // 获取短期上报曲线
        Map<Long, ForecastPowerShortTermRegulation> dqsbMap = new HashMap<>();
        QueryWrapper dqsbWrapper = new QueryWrapper<>();
        dqsbWrapper.eq("forecast_how_long_ago",Integer.parseInt(dqHowLongAgo));
        dqsbWrapper.eq("station_code",stationCode);
        dqsbWrapper.between("time", new Date(startTime), new Date(endTime));
        List<ForecastPowerShortTermRegulation> forecastPowerShortTermRegulationList =forecastPowerShortTermRegulationService.list(dqysWrapper);
        for (ForecastPowerShortTermRegulation forecastPowerShortTermRegulation : forecastPowerShortTermRegulationList) {
            dqsbMap.put(forecastPowerShortTermRegulation.getTime().getTime(), forecastPowerShortTermRegulation);
        }
        // 获取短期调控下发的中心曲线
        Map<Long, ForecastPowerShortTermSend> dqzxMap = new HashMap<>();
        QueryWrapper dqzxWrapper = new QueryWrapper<>();
        dqzxWrapper.eq("forecast_how_long_ago",Integer.parseInt(dqHowLongAgo));
        dqzxWrapper.eq("station_code",stationCode);
        dqzxWrapper.between("time", new Date(startTime), new Date(endTime));
        List<ForecastPowerShortTermSend> forecastPowerShortTermSendList =forecastPowerShortTermSendService.list(dqysWrapper);
        for (ForecastPowerShortTermSend forecastPowerShortTermSend : forecastPowerShortTermSendList) {
            dqzxMap.put(forecastPowerShortTermSend.getTime().getTime(), forecastPowerShortTermSend);
        }


        List<Map<String,Object>> tableList = new ArrayList();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        int i=0;
        for (Long tempDateTime = startTime; tempDateTime < endTime; tempDateTime = tempDateTime + 15 * 60 * 1000) {
            // 封装列表的数据
            Map<String,Object> map1 = new HashMap<>();
            map1.put("czmc", electricField.getName());
            map1.put("time", DateUtil.format(new Date(tempDateTime), "yyyy-MM-dd HH:mm"));

            // 封装实际功率list
            if (sjMap.get(tempDateTime) != null) {
                // 实际功率
                PowerStationStatusData powerStationStatusData = sjMap.get(tempDateTime);
                if (powerStationStatusData.getRealValue().compareTo(new BigDecimal("-99"))==0) {
                    sjList.add(null);
                    map1.put("sjgl",null);
                }
                else{
                    sjList.add(powerStationStatusData.getRealValue().toString());
                    map1.put("sjgl",powerStationStatusData.getRealValue().toString());
                }
                if (powerStationStatusData.getAbleValue().compareTo(new BigDecimal("-99"))==0) {
                    kyList.add(null);
                    map1.put("kygl",null);
                }
                else{
                    kyList.add(powerStationStatusData.getAbleValue().toString());
                    map1.put("kygl",powerStationStatusData.getAbleValue().toString());
                }
                if (powerStationStatusData.getTheoryValue().compareTo(new BigDecimal("-99"))==0) {
                    llList.add(null);
                    map1.put("llgl",null);
                }
                else{
                    llList.add(powerStationStatusData.getTheoryValue().toString());
                    map1.put("llgl",powerStationStatusData.getTheoryValue().toString());
                }
            } else {
                sjList.add(null);
                kyList.add(null);
                llList.add(null);
            }
            // 封装短期原始曲线list
            if (dqysMap.get(tempDateTime) !=null){
                ForecastPowerShortTermStation forecastPowerShortTermStation = dqysMap.get(tempDateTime);
                if (forecastPowerShortTermStation.getFpValue().compareTo(new BigDecimal("-99"))==0) {
                    dqysList.add(null);
                    map1.put("dqys",null);
                }
                else{
                    dqysList.add(forecastPowerShortTermStation.getFpValue().toString());
                    map1.put("dqys",forecastPowerShortTermStation.getFpValue().toString());
                }
            }
            else {
                dqysList.add(null);
            }
            // 封装短期上报曲线list
            if (dqsbMap.get(tempDateTime) !=null){
                ForecastPowerShortTermRegulation forecastPowerShortTermRegulation = dqsbMap.get(tempDateTime);
                if (forecastPowerShortTermRegulation.getFpValue().compareTo(new BigDecimal("-99"))==0) {
                    dqsbList.add(null);
                    map1.put("dqsb",null);
                }
                else{
                    dqsbList.add(forecastPowerShortTermRegulation.getFpValue().toString());
                    map1.put("dqsb",forecastPowerShortTermRegulation.getFpValue().toString());
                }
            }
            else {
                dqsbList.add(null);
            }
            // 封装短期中心曲线list
            if (dqzxMap.get(tempDateTime) !=null){
                ForecastPowerShortTermSend forecastPowerShortTermSend = dqzxMap.get(tempDateTime);
                if (forecastPowerShortTermSend.getFpValue().compareTo(new BigDecimal("-99"))==0) {
                    dqzxList.add(null);
                    map1.put("dqzx",null);
                }
                else{
                    dqzxList.add(forecastPowerShortTermSend.getFpValue().toString());
                    map1.put("dqzx",forecastPowerShortTermSend.getFpValue().toString());
                }
            }
            else {
                dqzxList.add(null);
            }


            // 封装时间轴list
            String time = format.format(tempDateTime);
            timeList.add(time);

            i++;
            tableList.add(map1);
        }


        Map<String, List> map = new HashMap<>();

        map.put("fromHead", headList);
        map.put("tableList", tableList);

        return map;
    }

    @Override
    public List<PowerStationStatusData> findByTimeBetweenAndStationCode(Date pointStarTime, Date pointEndTime, String stationCode) {
        QueryWrapper<PowerStationStatusData> wrapper = new QueryWrapper<>();

        if (stationCode != null && !stationCode.equals("")) {
            wrapper.eq("station_code", stationCode);
        }
        if (pointStarTime != null && pointEndTime != null) {
            wrapper.between("time", pointStarTime, pointEndTime);
        }
        return baseMapper.selectList(wrapper);
    }


    /**
     * 中心侧预测数据统计，查询短期、超短期实时表厂家是SYJY的，每个点位值累加一起展示
     * @param startTime
     * @param endTime
     * @return
     */
//    public Map<String, List> getCenterData(Date startTime, Date endTime) {
//        Long startDateTime = startTime.getTime();
//        Long endDateTime = endTime.getTime();
//
//        // 封装列表头信息
//        List headList = new ArrayList();
//        Map<String,String> timeHeadMap = new HashMap<>();
//        timeHeadMap.put("field", "time");
//        timeHeadMap.put("label", "时间");
//        headList.add(timeHeadMap);
//        Map<String,String> dqHeadMap = new HashMap<>();
//        dqHeadMap.put("field", "dq");
//        dqHeadMap.put("label", "短期(MW)");
//        headList.add(dqHeadMap);
//        Map<String,String> cdqHeadMap = new HashMap<>();
//        cdqHeadMap.put("field", "cdq");
//        cdqHeadMap.put("label", "超短期(MW)");
//        headList.add(cdqHeadMap);
//
//        // 获取短期数据SYJY
//        QueryWrapper<ForecastPowerShortTermCloud> wrapper = new QueryWrapper<>();
//        wrapper.eq("forecast_manufactor","SYJY");
//        wrapper.between("forecast_time", startTime, endTime);
//        List<ForecastPowerShortTermCloud> forecastPowerShortTermCloudList = forecastPowerShortTermService.list(wrapper);
////        Map<Long, List<ForecastPowerShortTermCloud>> shortGroupMap = forecastPowerShortTermCloudList.stream().collect(Collectors.groupingBy(s->s.getForecastTime().getTime()));
//
//        // 获取超短期数据SYJY
//        QueryWrapper<ForecastPowerUltraShortTermCloud> ultrawrapper = new QueryWrapper<>();
//        ultrawrapper.eq("forecast_manufactor","SYJY");
//        ultrawrapper.between("forecast_time", startTime, endTime);
//        List<ForecastPowerUltraShortTermCloud> forecastPowerUltraShortTermCloudList = forecastPowerUltraShortTermService.list(ultrawrapper);
////        Map<Long, List<ForecastPowerUltraShortTermCloud>> ultraGroupMap = forecastPowerUltraShortTermCloudList.stream().collect(Collectors.groupingBy(s->s.getForecastTime().getTime()));
//
//        List<String> timeList = new ArrayList<>();
//        List<String> dqList = new ArrayList();
//        List<String> cdqList = new ArrayList();
//
//        List<Map<String,Object>> tableList = new ArrayList();
//        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//        int i=0;
//        for (Long tempDateTime = startDateTime; tempDateTime < endDateTime; tempDateTime = tempDateTime + 15 * 60 * 1000) {
//            // 封装时间轴list
//            String time = format.format(tempDateTime);
//            timeList.add(time);
//            // 封装列表的数据
//            Map<String,Object> map1 = new HashMap<>();
//            map1.put("time", DateUtil.format(new Date(tempDateTime), "yyyy-MM-dd HH:mm"));
//            // 短期数据求和
//            BigDecimal dqBigDecimal = new BigDecimal(0);
//            if (shortGroupMap.get(tempDateTime)!=null){
//                List<ForecastPowerShortTermCloud> forecastPowerShortTermCloudList1 = shortGroupMap.get(tempDateTime);
//                for (ForecastPowerShortTermCloud forecastPowerShortTermCloud : forecastPowerShortTermCloudList1){
//                    dqBigDecimal = dqBigDecimal.add(forecastPowerShortTermCloud.getFpValue());
//                }
//                map1.put("dq",dqBigDecimal.toString());
//            }
//            else{
//                map1.put("dq",0);
//            }
//            dqList.add(dqBigDecimal.toString());
//
//            // 超短期数据求和
//            BigDecimal cdqBigDecimal = new BigDecimal(0);
//            if (ultraGroupMap.get(tempDateTime)!=null){
//                List<ForecastPowerUltraShortTermCloud> forecastPowerUltraShortTermCloudList1 = ultraGroupMap.get(tempDateTime);
//                for (ForecastPowerUltraShortTermCloud forecastPowerUltraShortTermCloud : forecastPowerUltraShortTermCloudList1){
//                    cdqBigDecimal = cdqBigDecimal.add(forecastPowerUltraShortTermCloud.getFpValue());
//                }
//                map1.put("cdq",cdqBigDecimal.toString());
//            }
//            else{
//                map1.put("cdq",0);
//            }
//            cdqList.add(cdqBigDecimal.toString());
//
//            tableList.add(map1);
//        }
//
//        Map<String, List> map = new HashMap<>();
//        map.put("dq", dqList);
//        map.put("cdq", cdqList);
//        map.put("time", timeList);
//        map.put("fromHead", headList);
//        map.put("tableList", tableList);
//
//        return map;
//    }
}
